==> em_tasks-0.1.35-py2.py3-none-any.whl: em_tasks/tasks/google_ads_performance_task.py <==
import json
import datetime

from google.oauth2 import service_account
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.oauth2 import _SERVICE_ACCOUNT_SCOPES

from em_tasks import logger
from em_tasks.store.models import *


class GoogleAdsPerformanceTask():
  def __init__(self, customer_id, developer_token, version='v15', date_range='last30'):
    if date_range not in ['last7', 'last14', 'last30']:
      date_range = 'last30'

    self.end_date = datetime.date.today() - datetime.timedelta(days=1)
    days = 30
    report_type = 3
    if date_range == 'last7':
      days = 7
      report_type = 1
    elif date_range == 'last14':
      days = 14
      report_type = 2
    else:
      days = 30
      report_type = 3
    self.start_date = self.end_date - datetime.timedelta(days=days)
    self.report_type = report_type

    ga_account = GoogleAdsAccount.get_or_none(GoogleAdsAccount.customer_id == customer_id)

    self.ga_id = ga_account.id
    self.impersonated_email = ga_account.impersonated_email
    self.customer_id = customer_id
    self.active = ga_account and ga_account.active

    service_account_info = json.loads(ga_account.service_account_data)
    credentials = service_account.Credentials.from_service_account_info(
      service_account_info,
      subject=self.impersonated_email,
      scopes=_SERVICE_ACCOUNT_SCOPES)
    self.ga_client = GoogleAdsClient(credentials, developer_token, version=version)
    self.ga_service = self.ga_client.get_service("GoogleAdsService")

  def run(self):
    headers = [
      'product_merchant_id', 'product_item_id',
      'impressions', 'clicks', 'ctr', 'average_cpc',
      'orders', 'units_sold', 'revenue_micros', 'cost_micros', 'average_order_value_micros',
      'conversions', 'conversions_value', 'cost_per_conversion', 'value_per_conversion',
      'all_conversions', 'all_conversions_value', 'cost_per_all_conversions', 'value_per_all_conversions',
      'search_impression_share', 'search_click_share'
    ]
    integer_fields = ['impressions', 'clicks', 'orders', 'units_sold', 'conversions', 'all_conversions']
    float_fields = [
      'ctr', 'conversions_value', 'value_per_conversion',
      'all_conversions_value', 'value_per_all_conversions',
      'search_impression_share', 'search_click_share'
    ]
    micro_fields = ['average_cpc', 'cost_per_conversion', 'cost_per_all_conversions']
    field_mappings = {
      'product_merchant_id': 'merchant_id',
      'product_item_id': 'item_id',
      'revenue_micros': 'revenue',
      'cost_micros': 'cost',
      'average_order_value_micros': 'average_order_value'
    }
    batch_size = 5000
    today_str = datetime.date.today().strftime('%Y-%m-%d')

    performances = dict()

    where_query = 'WHERE segments.date BETWEEN "{}" AND "{}"'.format(
        self.start_date.strftime('%Y-%m-%d'), self.end_date.strftime('%Y-%m-%d'))
    # where_query += ' AND metrics.clicks > 0'
    logger.debug('[WhereQuery] %s', where_query)

    query = """
      SELECT
        segments.product_merchant_id,
        segments.product_item_id,
        metrics.impressions,
        metrics.clicks,
        metrics.ctr,
        metrics.average_cpc,
        metrics.orders,
        metrics.units_sold,
        metrics.revenue_micros,
        metrics.cost_micros,
        metrics.average_order_value_micros,
        metrics.conversions,
        metrics.conversions_value,
        metrics.cost_per_conversion,
        metrics.value_per_conversion,
        metrics.all_conversions,
        metrics.all_conversions_value,
        metrics.cost_per_all_conversions,
        metrics.value_per_all_conversions,
        metrics.search_impression_share,
        metrics.search_click_share
      FROM  shopping_performance_view
      {}
      ORDER BY
        metrics.all_conversions DESC,
        metrics.conversions DESC,
        metrics.clicks DESC,
        metrics.cost_micros DESC,
        metrics.impressions DESC""".format(where_query)

    # Issues a search request using streaming.
    search_request = self.ga_client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = self.customer_id
    search_request.query = query
    try:
      stream = self.ga_service.search_stream(search_request)
      for batch in stream:
        for row in batch.results:
          segments = row.segments
          metrics = row.metrics

          item = []
          performance = {'google_ads_account_id': self.ga_id, 'type': self.report_type, 'last_update_date': today_str}
          for header in headers:
            val = getattr(segments, header, getattr(metrics, header, ''))
            if header in integer_fields:
              val = int(val)
            elif header in float_fields:
              val = round(float(val), 2)
            elif header.find('_micros') != -1 or header in micro_fields:
              val = round(float(val) / 1000000, 2)
            else:
              pass

            if header in field_mappings:
              performance[field_mappings[header]] = val
            else:
              performance[header] = val

          if performance['merchant_id'] not in performances:
            performances[performance['merchant_id']] = dict()
          performances[performance['merchant_id']][performance['item_id']] = performance

          for merchant_id, items in performances.items():
            if len(items) < batch_size:
              continue

            self.process_performance_items(merchant_id, items)
            performances[merchant_id].clear()

      for merchant_id, items in performances.items():
        if not items:
          continue

        self.process_performance_items(merchant_id, items)
        performances[merchant_id].clear()
        
    except GoogleAdsException as ex:
      message = 'Request with ID "{}" failed with status "{}" and includes the following errors:'.format(
        ex.request_id, ex.error.code().name)

      errors = []
      for error in ex.failure.errors:
        error_msg = 'Error with message {}.'.format(error.message)
        if error.location:
          error_msg += "\n"
          for field_path_element in error.location.field_path_elements:
            error_msg += "On field: {}\n".format(field_path_element.field_name)

        errors.append(error_msg)

      logger.error(message + "\n" + "\n".join(errors))

  def process_performance_items(self, merchant_id, items):
    preserves = (
      GoogleAdsPerformance.impressions,
      GoogleAdsPerformance.clicks,
      GoogleAdsPerformance.ctr,
      GoogleAdsPerformance.average_cpc,
      GoogleAdsPerformance.orders,
      GoogleAdsPerformance.units_sold,
      GoogleAdsPerformance.revenue,
      GoogleAdsPerformance.cost,
      GoogleAdsPerformance.average_order_value,
      GoogleAdsPerformance.conversions,
      GoogleAdsPerformance.conversions_value,
      GoogleAdsPerformance.cost_per_conversion,
      GoogleAdsPerformance.value_per_conversion,
      GoogleAdsPerformance.roas,
      GoogleAdsPerformance.all_conversions,
      GoogleAdsPerformance.all_conversions_value,
      GoogleAdsPerformance.cost_per_all_conversions,
      GoogleAdsPerformance.value_per_all_conversions,
      GoogleAdsPerformance.search_impression_share,
      GoogleAdsPerformance.search_click_share
    )
    to_create = []
    to_update = []
    item_ids = list(items.keys())
    ga_performances = dict()
    for ga_performance in GoogleAdsPerformance.select().where((GoogleAdsPerformance.google_ads_account_id == self.ga_id) & (GoogleAdsPerformance.type == self.report_type) & (GoogleAdsPerformance.merchant_id == merchant_id) & GoogleAdsPerformance.item_id.in_(item_ids)).execute():
      ga_performances[ga_performance.item_id] = ga_performance

    for item_id, item in items.items():
      if item['cost'] <= 0:
        item['roas'] = 0
      else:
        item['roas'] = round(item['conversions_value'] / item['cost'], 2)
      if item_id in ga_performances:
        item['id'] = ga_performances[item_id].id
        to_update.append(item)
      else:
        to_create.append(item)

    if to_update:
      GoogleAdsPerformance.insert_many(to_update).on_conflict(preserve=preserves).execute()

    if to_create:
      GoogleAdsPerformance.insert_many(to_create).execute()

==> em_tasks-0.1.57-py2.py3-none-any.whl: em_tasks/tasks/google_ads_performance_task.py <==
import json
import datetime

from google.oauth2 import service_account
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.oauth2 import _SERVICE_ACCOUNT_SCOPES

from em_tasks import logger
from em_tasks.store.models import *


class GoogleAdsPerformanceTask():
  def __init__(self, customer_id, developer_token, version='v15', date_range='last30'):
    # if date_range not in ['last7', 'last14', 'last30']:
    #   date_range = 'last30'

    self.end_date = datetime.date.today() - datetime.timedelta(days=1)
    date_range = 'last30'
    days = 30
    report_type = 3
    # if date_range == 'last7':
    #   days = 7
    #   report_type = 1
    # elif date_range == 'last14':
    #   days = 14
    #   report_type = 2
    # else:
    #   days = 30
    #   report_type = 3
    self.start_date = self.end_date - datetime.timedelta(days=days)
    self.report_type = report_type

    ga_account = GoogleAdsAccount.get_or_none(GoogleAdsAccount.customer_id == customer_id)

    self.ga_id = ga_account.id
    self.impersonated_email = ga_account.impersonated_email
    self.customer_id = customer_id
    self.active = ga_account and ga_account.active

    service_account_info = json.loads(ga_account.service_account_data)
    credentials = service_account.Credentials.from_service_account_info(
      service_account_info,
      subject=self.impersonated_email,
      scopes=_SERVICE_ACCOUNT_SCOPES)
    self.ga_client = GoogleAdsClient(credentials, developer_token, version=version)
    self.ga_service = self.ga_client.get_service("GoogleAdsService")

  def run(self):
    headers = [
      'product_merchant_id', 'product_item_id',
      'impressions', 'clicks', 'ctr', 'average_cpc',
      'orders', 'units_sold', 'revenue_micros', 'cost_micros', 'average_order_value_micros',
      'conversions', 'conversions_value', 'cost_per_conversion', 'value_per_conversion',
      'all_conversions', 'all_conversions_value', 'cost_per_all_conversions', 'value_per_all_conversions',
      'search_impression_share', 'search_click_share'
    ]
    integer_fields = ['impressions', 'clicks', 'orders', 'units_sold', 'conversions', 'all_conversions']
    float_fields = [
      'ctr', 'conversions_value', 'value_per_conversion',
      'all_conversions_value', 'value_per_all_conversions',
      'search_impression_share', 'search_click_share'
    ]
    micro_fields = ['average_cpc', 'cost_per_conversion', 'cost_per_all_conversions']
    field_mappings = {
      'product_merchant_id': 'merchant_id',
      'product_item_id': 'item_id',
      'revenue_micros': 'revenue',
      'cost_micros': 'cost',
      'average_order_value_micros': 'average_order_value'
    }
    batch_size = 5000
    today_str = datetime.date.today().strftime('%Y-%m-%d')

    performances = dict()

    where_query = 'WHERE segments.date BETWEEN "{}" AND "{}"'.format(
        self.start_date.strftime('%Y-%m-%d'), self.end_date.strftime('%Y-%m-%d'))
    # where_query += ' AND metrics.clicks > 0'
    logger.debug('[WhereQuery] %s', where_query)

    query = """
      SELECT
        segments.product_merchant_id,
        segments.product_item_id,
        metrics.impressions,
        metrics.clicks,
        metrics.ctr,
        metrics.average_cpc,
        metrics.orders,
        metrics.units_sold,
        metrics.revenue_micros,
        metrics.cost_micros,
        metrics.average_order_value_micros,
        metrics.conversions,
        metrics.conversions_value,
        metrics.cost_per_conversion,
        metrics.value_per_conversion,
        metrics.all_conversions,
        metrics.all_conversions_value,
        metrics.cost_per_all_conversions,
        metrics.value_per_all_conversions,
        metrics.search_impression_share,
        metrics.search_click_share
      FROM  shopping_performance_view
      {}
      ORDER BY
        metrics.all_conversions DESC,
        metrics.conversions DESC,
        metrics.clicks DESC,
        metrics.cost_micros DESC,
        metrics.impressions DESC""".format(where_query)

    # Issues a search request using streaming.
    search_request = self.ga_client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = self.customer_id
    search_request.query = query
    try:
      stream = self.ga_service.search_stream(search_request)
      for batch in stream:
        for row in batch.results:
          segments = row.segments
          metrics = row.metrics

          item = []
          performance = {'google_ads_account_id': self.ga_id, 'last_update_date': today_str}
          for header in headers:
            val = getattr(segments, header, getattr(metrics, header, ''))
            if header in integer_fields:
              val = int(val)
            elif header in float_fields:
              val = round(float(val), 2)
            elif header.find('_micros') != -1 or header in micro_fields:
              val = round(float(val) / 1000000, 2)
            else:
              pass

            if header in field_mappings:
              performance[field_mappings[header]] = val
            else:
              performance[header] = val

          if performance['merchant_id'] not in performances:
            performances[performance['merchant_id']] = dict()
          performances[performance['merchant_id']][performance['item_id']] = performance

          for merchant_id, items in performances.items():
            if len(items) < batch_size:
              continue

            self.process_performance_items(merchant_id, items)
            performances[merchant_id].clear()

      for merchant_id, items in performances.items():
        if not items:
          continue

        self.process_performance_items(merchant_id, items)
        performances[merchant_id].clear()
        
    except GoogleAdsException as ex:
      message = 'Request with ID "{}" failed with status "{}" and includes the following errors:'.format(
        ex.request_id, ex.error.code().name)

      errors = []
      for error in ex.failure.errors:
        error_msg = 'Error with message {}.'.format(error.message)
        if error.location:
          error_msg += "\n"
          for field_path_element in error.location.field_path_elements:
            error_msg += "On field: {}\n".format(field_path_element.field_name)

        errors.append(error_msg)

      logger.error(message + "\n" + "\n".join(errors))

  def process_performance_items(self, merchant_id, items):
    preserves = (
      GoogleAdsPerformance.impressions,
      GoogleAdsPerformance.clicks,
      GoogleAdsPerformance.ctr,
      GoogleAdsPerformance.average_cpc,
      GoogleAdsPerformance.orders,
      GoogleAdsPerformance.units_sold,
      GoogleAdsPerformance.revenue,
      GoogleAdsPerformance.cost,
      GoogleAdsPerformance.average_order_value,
      GoogleAdsPerformance.conversions,
      GoogleAdsPerformance.conversions_value,
      GoogleAdsPerformance.cost_per_conversion,
      GoogleAdsPerformance.value_per_conversion,
      GoogleAdsPerformance.roas,
      GoogleAdsPerformance.all_conversions,
      GoogleAdsPerformance.all_conversions_value,
      GoogleAdsPerformance.cost_per_all_conversions,
      GoogleAdsPerformance.value_per_all_conversions,
      GoogleAdsPerformance.search_impression_share,
      GoogleAdsPerformance.search_click_share
    )
    to_create = []
    to_update = []
    item_ids = list(items.keys())
    ga_performances = dict()
    for ga_performance in GoogleAdsPerformance.select().where((GoogleAdsPerformance.google_ads_account_id == self.ga_id) & (GoogleAdsPerformance.type == self.report_type) & (GoogleAdsPerformance.merchant_id == merchant_id) & GoogleAdsPerformance.item_id.in_(item_ids)).execute():
      ga_performances[ga_performance.item_id] = ga_performance

    for item_id, item in items.items():
      if item['cost'] <= 0:
        item['roas'] = 0
      else:
        item['roas'] = round(item['conversions_value'] / item['cost'], 2)
      if item_id in ga_performances:
        item['id'] = ga_performances[item_id].id
        to_update.append(item)
      else:
        to_create.append(item)

    if to_update:
      GoogleAdsPerformance.insert_many(to_update).on_conflict(preserve=preserves).execute()

    if to_create:
      GoogleAdsPerformance.insert_many(to_create).execute()

    logger.debug('[ItemsProcessed] %s', len(items))
